from StoppableThread import StoppableThread
from threading import Thread, Event

import array
import sys
import colorsys

sys.path.insert(0, '/Users/lay/workspace/dmx-lib/muse_controller/nanoleafpy2')
from nanoleaf import setup, Aurora

from multiprocessing import Pool

ANIMATION_ID = "myanimation"

class NanoleafLightManager:
    def __init__(self):

        # TODO this shit needs to be automated
        ipAuthMap = {
            u'192.168.128.31': u'jcjYtfzt4zusn82lSJ5eHLOPzuiOgoDP',
            u'192.168.128.32': u'WTq3eIKY8fdfgGZjQlvbbe7E7fkKYDW4'
        }
        print "Instatiating Aurora Objects..."
        self.auroras = []
        for ip in ipAuthMap:
            print "ip:", ip
            print "token:", ipAuthMap[ip]
            self.auroras += [{'aurora': Aurora(ip, ipAuthMap[ip])}]
        print "Finished Instatiating Aurora Objects..."

        print "Initiating Aurora Metadata..."
        for aurora in self.auroras:
            aurora['panelIDs'] = [x['panelId'] for x in aurora['aurora'].panel_positions]
        print "Finished Initiating Aurora Metadata..."

        print "Turning on Auroras"
        for aurora in self.auroras:
            aurora['aurora'].on = True
            # aurora['aurora'].effect = ANIMATION_ID
        print "Finished Turning on Auroras"

        # self.pool = Pool(processes=2) # Start a worker processes.

# +++++++++++++++++++++++
        # print "Retrieving auroras..."
        # ipAddressList = setup.find_auroras()
        # print "Auroras found:", ipAddressList
        #
        # # print "Selecting only whitelisted auroras..."
        # # ipWhitelist = []
        # # ipAddressList = [x for x in ipAddressList if x in ipWhitelist]
        # # print "Auroras used:", ipAddressList
        #
        # print "Retrieving auth tokens..."
        # ipAuthMap = {}
        # for ip in ipAddressList:
        #     ipAuthMap[ip] = setup.generate_auth_token(ip)
        # print "Tokens generated:", ipAuthMap
        #
        # print "Instatiating Aurora Objects..."
        # self.auroras = []
        # for ip in ipAuthMap:
        #     self.auroras += [{'aurora': Aurora(ip, ipAuthMap[ip])}]
        # print "Finished Instatiating Aurora Objects..."
        #
        # print "Initiating Aurora Metadata..."
        # self.auroras = {}
        # for aurora in self.auroras:
        #     aurora['panelIDs'] = [x.panelId for x in self.aurora.panel_positions()]
        # print "Finished Initiating Aurora Metadata..."
        #
        # print "Turning on Auroras"
        # for aurora in self.auroras:
        #     aurora['aurora'].on = True
        #     # aurora['aurora'].effect = ANIMATION_ID
        # print "Finished Turning on Auroras"
        #
        # print "Setting lights to white"
        # self.updateLights(255, 255, 255, 100)
        # print "Finished setting lights to white"

    def updateAurora(self, aurora, effect):
        print("Updating aurora {}".format(aurora))
        print("Response", aurora['aurora'].effect_set_raw(effect))
        return 0

    def updateLights(self, r, g, b, brightness):
        # TODO Modify RGB values by brightness. Experimental:
        r = int(r * (brightness / 255.0))
        g = int(g * (brightness / 255.0))
        b = int(b * (brightness / 255.0))

        threads = []
        for aurora in self.auroras:
            try:
                effect = self.getStaticEffect(aurora, r,g,b)
                t = Thread(target=self.updateAurora, args=(aurora, effect,))
                # t.setDaemon(True)
                threads.append(t)
                # lightServerThreadNanoleaf = StoppableThread(self.updateAurora)
                # lightServerThreadNanoleaf.start()
                # aurora['aurora'].effect_set_raw(effect)

                # result = self.pool.apply_async(self.updateAurora, [aurora, effect])
                # print "result", result, result.get()

            except Exception, err:
                print "Exception in NanoleafClient.updateLights(): ", err.__class__.__name__, err.message

        for thread in threads:
            thread.start()
        for thread in threads:
            thread.join()

    def getStaticEffect(self, aurora, r, g, b):
        panelIDs = aurora['panelIDs']
        numFrames = 1
        transitionTime = 1 # Decaseconds

        # animData is of the form: <numPanels> <panelId0> <numFrames0> <RGBWT01> <RGBWT02> ... <RGBWT0n(0)> <panelId1> <numFrames1> <RGBWT11> <RGBWT12> ... <RGBWT1n(1)> ... ... <panelIdN> <numFramesN> <RGBWTN1> <RGBWTN2> ... <RGBWTNn(N)>
        animData = "%d " % (len(panelIDs))
        for panelID in panelIDs:
            animData += "%d %d %d %d %d 0 %d " % (panelID, numFrames, r, g, b, transitionTime)

        return {
            "command": "add",
            "animName": ANIMATION_ID,
            "animType": "static",
            "animData": animData,
            "loop": False
        }

    def kill(self):
        pass



class MuseState(object):
    def __init__(self):
        # waves
        self.alpha = 0
        self.beta = 0
        self.delta = 0
        self.gamma = 0
        self.theta = 0
        # is the muse on the users head?
        self.touching_forehead = 0
        # 'connected' is a computed value. see horseshoe_callback() for implementation
        self.connected = 0
        # The current level of connectivity. 0-1
        self.connectionScore = 0



from threading import Thread, Event

# https://stackoverflow.com/questions/323972/is-there-any-way-to-kill-a-thread-in-python
class StoppableThread(Thread):
    """Thread class with a stop() method. The thread itself has to check
    regularly for the stopped() condition."""

    def __init__(self, target):
        super(StoppableThread, self).__init__(target=target, args=(self,))
        self.target = target
        self._stop_event = Event()

    def stop(self):
        self._stop_event.set()

    def stopped(self):
        return self._stop_event.is_set()



from collections import deque
import math

class MovingAverageExponential(object):
    def __init__(self, size):
        """
        Initialize your data structure here.
        :type size: int
        """
        self.__size = size if size >= 1 else 1
        self.__sum = 0
        self.__q = deque([])
        self.__exponential_backoff = .97
    def next(self, val, printDebug = False):
        """
        :type val: int
        :rtype: float
        """
        val = val if not math.isnan(val) else 0
        if len(self.__q) >= self.__size:
            subVal = self.__q.popleft()
            self.__sum -= self.__exponential_backoff**(len(self.__q)-1)*subVal
        self.__sum = (self.__sum*self.__exponential_backoff) + val
        self.__q.append(val)
        return max(min(1.0 * self.__sum / self.computeBackoffMass(len(self.__q)),1),0)

    def computeBackoffMass(self, n):
        return self.__exponential_backoff*(1-self.__exponential_backoff**(n))/(1-self.__exponential_backoff)

# https://github.com/kamyu104/LeetCode/blob/master/Python/moving-average-from-data-stream.py
# Not weighted
class MovingAverageLinear(object):
    def __init__(self, size):
        """
        Initialize your data structure here.
        :type size: int
        """
        self.__size = size
        self.__sum = 0
        self.__q = deque([])

    def next(self, val, printDebug = False):
        """
        :type val: int
        :rtype: float
        """
        if len(self.__q) == self.__size:
            self.__sum -= self.__q.popleft()
        self.__sum += val
        self.__q.append(val)
        return 1.0 * self.__sum / len(self.__q)



import pytweening
import random
import colorsys
import time
import sys
import numpy as np
from random import shuffle
from MuseState import MuseState
from MovingAverage import MovingAverageExponential, MovingAverageLinear
from StoppableThread import StoppableThread
from Config import LIGHT_UPDATE_INTERVAL

class LightState:
    def __init__(self):
        self.r = 0
        self.g = 0
        self.b = 0
        self.brightness = 0

def ease(easingFunc, old_value, new_value, current_increment, final_increment):
    percentComplete = abs(float(current_increment) / float(final_increment))
    diff = (old_value - new_value)
    increment = diff * easingFunc(percentComplete)
    return int(old_value - increment)

def softmax(x):
    """Compute softmax values for each sets of scores in x."""
    return x / np.sum(x, axis=0)

def exponentialSoftmax(rgb):
    # return rgb
    output = list(softmax(map(lambda x: x**2, rgb)))
    return output

class LightMixer():
    def updateState(self, user_state):
        raise NotImplementedError('Need to implement updateState')
    def getLight(self):
        raise NotImplementedError('Need to implement getLight')
    def kill(self):
        raise NotImplementedError('Need to implement kill')

class EEGWaveLightMixer(LightMixer):
    def __init__(self,
        user_to_default_fade_window,
        default_animation_render_rate,
        default_animation_brightness,
        user_light_brightness):
        self.default_animation_render_rate = default_animation_render_rate
        self.default_animation_brightness = default_animation_brightness
        self.user_light_brightness = user_light_brightness

        # These values help use keep track of when the user is connected to the muse
        self.connected_mean = 0
        self.touching_forehead_mean = 0
        self.connected_rolling_mean_generator = MovingAverageLinear(user_to_default_fade_window)
        self.touching_forehead_mean_generator = MovingAverageLinear(user_to_default_fade_window)

        # Muse data, and user info
        self.userState = MuseState()

        # Smooth the EEG data to 100fps
        # 0.1 is the muse update interval
        smoothingRange = 0.1 / LIGHT_UPDATE_INTERVAL
        movingAverageChoice = MovingAverageExponential#MovingAverageLinear
        self.alphaRollingAvgGenerator = movingAverageChoice(smoothingRange)
        self.betaRollingAvgGenerator = movingAverageChoice(smoothingRange)
        self.deltaRollingAvgGenerator = movingAverageChoice(smoothingRange)
        self.gammaRollingAvgGenerator = movingAverageChoice(smoothingRange)
        self.thetaRollingAvgGenerator = movingAverageChoice(smoothingRange)

        # The data that represents the user light
        self.userLight = LightState()
        # The current colour of the default animation
        self.defaultLight = LightState()
        # The weighted mix of the user and default animation color.
        # When the user connects (to the muse) this color will transition
        # over 3 seconds to be their color, whe the user disconnects, this color
        # transitions to the default animation color
        self.mixedLight = LightState()

        # We use these as the rolling avarage of the equivalent user values over the last 100 milliseconds.
        # This way we can get EEG data (and thus the lights animating them) to
        # run at 100fps, instead of 10
        self.rollingUserAlpha = 0
        self.rollingUserBeta = 0
        self.rollingUserDelta = 0
        self.rollingUserGamma = 0
        self.rollingUserTheta = 0

        self.easeEEGValuesThread = StoppableThread(self.easeEEGValues)
        self.easeEEGValuesThread.start()

    def easeEEGValues(self, thread):
        while not thread.stopped():
            self.rollingUserAlpha = self.alphaRollingAvgGenerator.next(self.userState.alpha)
            self.rollingUserBeta = self.betaRollingAvgGenerator.next(self.userState.beta)
            self.rollingUserDelta = self.deltaRollingAvgGenerator.next(self.userState.delta)
            self.rollingUserGamma = self.gammaRollingAvgGenerator.next(self.userState.gamma)
            self.rollingUserTheta = self.thetaRollingAvgGenerator.next(self.userState.theta)
            time.sleep(LIGHT_UPDATE_INTERVAL)
        sys.exit()

    def startEasingEEGValues(self):
        self.defaultAnimationThread = StoppableThread(self.serveDefaultAnimation, )
        self.defaultAnimationThread.start()

    def startDefaultAnimation(self):
        self.defaultAnimationThread = StoppableThread(self.serveDefaultAnimation, )
        self.defaultAnimationThread.start()

    def serveDefaultAnimation(self, thread):
        timeToNextColor = 0
        currentTime = 0
        r,g,b = 0,0,0 # Starting color. Black
        self.defaultLight.brightness = self.default_animation_brightness
        oneTimeFadeIn = 0
        minRGBSum = 255
        MaxRGBSum = 510
        while not thread.stopped():
            if currentTime == timeToNextColor:
                r_old,g_old,b_old = r,g,b
                # https://stackoverflow.com/questions/43437309/get-a-bright-random-colour-python
                #Idea, all colours are 255 of a primary colour + some combo of 255 for the other two
                colourArray = [0]*3
                primaryIndex = random.randint(0, 2)
                colourArray[primaryIndex] = 255
                remainingIdx = list(set(list(range(len(colourArray)))) - set([primaryIndex]))
                shuffle(remainingIdx)
                secondaryIndex = remainingIdx[0]
                colourToAdd = random.randint(0, 255)
                colourArray[secondaryIndex] = colourToAdd
                print("Colour array {}".format(colourArray))
                r, g, b = colourArray
                timeToNextColor = random.randint(4/self.default_animation_render_rate,6/self.default_animation_render_rate)
                currentTime = 0
            easingFunction = pytweening.easeInOutQuad
            self.defaultLight.r = ease(easingFunction, r_old, r, currentTime, timeToNextColor)
            self.defaultLight.g = ease(easingFunction, g_old, g, currentTime, timeToNextColor)
            self.defaultLight.b = ease(easingFunction, b_old, b, currentTime, timeToNextColor)

            # Fade in the lights when we start the server
            if oneTimeFadeIn <= 1:
                # Fade lights in over 2 second
                oneTimeFadeIn += 0.5 * self.default_animation_render_rate

            # Dim the animation when the user puts on the muse
            # Dim to a minimum of 20% of the original animation brightness
            # TODO (touching forehead api is not working. find out why)
            # brightnessModifier = 1-self.touching_forehead_mean
            # brightnessModifier = max(0.2, brightnessModifier)
            # self.defaultLight.brightness = self.default_animation_brightness * (1-self.touching_forehead_mean)

            self.defaultLight.brightness = self.default_animation_brightness * oneTimeFadeIn
            currentTime += 1
            time.sleep(self.default_animation_render_rate)
        sys.exit()

    # This mixes the use and default colours depending on if the user is connected or not
    def updateMixedLight(self):
        e=self.mixedLight
        self.mixedLight.r = int((self.userLight.r * self.connected_mean) + (self.defaultLight.r * (1-self.connected_mean)))
        self.mixedLight.g = int((self.userLight.g * self.connected_mean) + (self.defaultLight.g * (1-self.connected_mean)))
        self.mixedLight.b = int((self.userLight.b * self.connected_mean) + (self.defaultLight.b * (1-self.connected_mean)))
        self.mixedLight.brightness = int((self.userLight.brightness * self.connected_mean) + (self.defaultLight.brightness * (1-self.connected_mean)))

    # interprets user state as a color
    def updateUserLight(self):
        # raw values are between 0 and 1. map it to 0-255
        lightBeta, lightGamma, lightAlpha = exponentialSoftmax([self.rollingUserBeta, self.rollingUserGamma, self.rollingUserAlpha])
        if np.isnan(lightBeta) or np.isnan(lightGamma) or np.isnan(lightAlpha):
            return
        self.userLight.r = lightBeta * 255
        self.userLight.g = lightGamma * 255
        self.userLight.b = lightAlpha * 255
        self.userLight.brightness = self.user_light_brightness

    # This function can be asynced if need be
    def updateState(self, user_state):
        self.userState = user_state
        self.connected_mean = self.connected_rolling_mean_generator.next(user_state.connected)
        self.touching_forehead_mean = self.touching_forehead_mean_generator.next(user_state.touching_forehead)
        self.updateUserLight()
        self.updateMixedLight()

    def getLight(self):
        return self.mixedLight

    def kill(self):
        self.easeEEGValuesThread.stop()
        if hasattr(self, 'defaultAnimationThread'):
            self.defaultAnimationThread.stop()



from liblo import *
from threading import Thread

import math
import numpy as np
import random
import sys
import time
import traceback
import urllib2

from LightManager import DMXLightManager, NanoleafLightManager
from LightMixer import EEGWaveLightMixer
from MovingAverage import MovingAverageExponential, MovingAverageLinear
from StoppableThread import StoppableThread
from MuseState import MuseState
import Config as config

# TODO make the nanoleafpy a propers dependency. Or just rip parts out of it
# TODO make the connection to nanoleaf an option. when you run the program there
# should be an option to search for and connect to existing nanoleaf installations
# Then it should store the ips and auth tokent for these installations in some
# persistant store.
# If the option is not specified, this program should try to read from said persistant store
#
# TODO Finish gutting DMX logic, and any extra abstraction that is no longer needed
# Make the threading a little easier to use?

def avg(*values):
    return reduce(lambda x, y: x + y, values) / len(values)

# Wrapper for NanoleafLightManager
class NanoleafClient():
    def __init__(self):
        self.lightManager = NanoleafLightManager()

    def updateLights(self, color):
        self.lightManager.updateLights(color.r, color.g, color.b, color.brightness)

    def kill(self):
        self.lightManager.kill()

receivingMessages = True
timeSinceLastSecond = 1
class MuseServer(ServerThread):

    def __init__(self):
        # Listen on port 5001
        ServerThread.__init__(self, 5001)
        MovingAverageChoice = MovingAverageExponential
        self.alpha_relative_rolling_avg_generator = MovingAverageChoice(config.ROLLING_EEG_WINDOW)
        self.beta_relative_rolling_avg_generator = MovingAverageChoice(config.ROLLING_EEG_WINDOW)
        self.delta_relative_rolling_avg_generator = MovingAverageChoice(config.ROLLING_EEG_WINDOW)
        self.gamma_relative_rolling_avg_generator = MovingAverageChoice(config.ROLLING_EEG_WINDOW)
        self.theta_relative_rolling_avg_generator = MovingAverageChoice(config.ROLLING_EEG_WINDOW)

        self.all_contacts_mean = MovingAverageChoice(config.CONTACT_LOS_TIMEOUT)

        self.state = MuseState()

        self.connections_debug = (0, 0, 0, 0)

        self.lightServerThreadNanoleaf = StoppableThread(self.serveNanoleafLights)
        self.lightServerThreadNanoleaf.start()
        self.globalMixer = None

        # Fake input
        self.state.alpha = .32
        self.state.beta = .32
        self.state.delta = .32
        self.state.gamma = .32
        self.state.theta = .32
        self.connectThread = StoppableThread(self.connectToggle)
        self.connectThread.start()

    def connectToggle(self, thread):
        self.state.connected = 1
        self.state.touching_forehead = 1
        while not thread.stopped():
            self.state.alpha = random.random()
            self.state.beta = random.random()
            self.state.delta = random.random()
            self.state.gamma = random.random()
            self.state.theta = random.random()
            self.state.connected = abs(1-self.state.connected)
            self.state.touching_forehead = abs(1-self.state.touching_forehead)
            print( "connected", self.state.connected)
            time.sleep(10)
        sys.exit()

    def kill(self):
        self.lightServerThreadNanoleaf.stop()
        self.connectThread.stop()

    def serveNanoleafLights(self, thread):
        nanoleafClient = NanoleafClient()

        while not thread.stopped():
            try:
                light = self.globalMixer.getLight()
                nanoleafClient.updateLights(light)
                # print "Nanoleaf lights (from Mixer), COLORS: r: %d g: %d b: %d, BRIGHTNESS: %d" % (light.r, light.g, light.b, light.brightness)
                time.sleep(config.NANOLEAF_LIGHT_UPDATE_INTERVAL)

            except Exception, err:
                print "Exception in serveNanoleafLights: ", err.__class__.__name__, err.message
                nanoleafClient.kill()
                sys.exit()
        print "EXITING serveNanoleafLights"
        nanoleafClient.kill()
        sys.exit()

    # Dim the light over 2 seconds. Then as a precaution set their colors to black
    def dimLights(self, dmxClient):
        # number of decaseconds to fade
        fadeCount = 2000

        startEEGBright = dmxClient.lightManager.getLightGroup(config.EEG_LIGHT_GROUP_ADDRESS).brightness
        curEEGBright = startEEGBright
        startSpotBright = dmxClient.lightManager.getLightGroup(config.SPOTLIGHT_LIGHT_GROUP_ADDRESS).brightness
        curSpotBright = startSpotBright

        for _ in range(fadeCount):
            curEEGBright -= config.USER_LIGHT_BRIGHTNESS / float(fadeCount)
            curEEGBright = max(0,int(curEEGBright))
            dmxClient.updateLightGroupBrightness(config.EEG_LIGHT_GROUP_ADDRESS,curEEGBright)
            curSpotBright -= config.DEFAULT_SPOTLIGHT_ANIMATION_BRIGHTNESS / float(fadeCount)
            curSpotBright = max(0, int(curSpotBright))
            dmxClient.updateLightGroupBrightness(config.SPOTLIGHT_LIGHT_GROUP_ADDRESS, curSpotBright)
            if curEEGBright == curSpotBright ==  0:
                break
            time.sleep(0.01)

    # receive alpha data
    @make_method('/muse/elements/alpha_relative', 'ffff')
    def alpha_relative_callback(self, path, *args):
        weights = np.array([1,1])
        values = np.array(args[0])
        values = np.array(values[np.argsort(values)[-2:]])
        alphaWeightedAndNormalized = self.weighter(values, weights, normalizationMin = .05, normalizationMax=.5)
        x = self.alpha_relative_rolling_avg_generator.next(alphaWeightedAndNormalized)
        self.state.alpha = x if not math.isnan(x) else 0

    # receive beta data
    @make_method('/muse/elements/beta_relative', 'ffff')
    def beta_relative_callback(self, path, *args):
        weights = np.array([1,1])
        values = np.array(args[0])
        values = np.array(values[np.argsort(values)[-2:]])
        betaWeightedAndNormalized = self.weighter(values, weights, normalizationMin=.05, normalizationMax=.5)
        x = self.beta_relative_rolling_avg_generator.next(betaWeightedAndNormalized)
        self.state.beta = x if not math.isnan(x) else 0

    # receive gamma data
    @make_method('/muse/elements/gamma_relative', 'ffff')
    def gamma_relative_callback(self, path, *args):
        weights = [1]*4
        values = args[0]
        gammaWeightedAndNormalized = self.weighter(values, weights)
        x = self.gamma_relative_rolling_avg_generator.next(gammaWeightedAndNormalized)
        self.state.gamma = x if not math.isnan(x) else 0

    # receive delta data
    @make_method('/muse/elements/delta_relative', 'ffff')
    def delta_relative_callback(self, path, *args):
        weights = [1]*4
        values = args[0]
        deltaWeightedAndNormalized = self.weighter(values, weights)
        x = self.delta_relative_rolling_avg_generator.next(deltaWeightedAndNormalized)
        self.state.delta = x if not math.isnan(x) else 0

    # receive theta data
    @make_method('/muse/elements/theta_relative', 'ffff')
    def theta_relative_callback(self, path, *args):
        weights = [1]*4
        values = args[0]
        thetaWeightedAndNormalized = self.weighter(values, weights)
        x = self.theta_relative_rolling_avg_generator.next(thetaWeightedAndNormalized)
        self.state.theta = x if not math.isnan(x) else 0


    def weighter(self, values, weights, normalizationMin = 0, normalizationMax = 1):
        avgVal = np.nanmean(np.multiply(values,weights))
        if np.isnan(avgVal):
            return 0
        normalizedAvgVal = (avgVal - normalizationMin) / (normalizationMax - normalizationMin)
        normalizedAvgVal = max(0, normalizedAvgVal)
        normalizedAvgVal = min(1, normalizedAvgVal)
        return normalizedAvgVal

    @make_method('/muse/elements/touching_forehead', 'i')
    def horseshoe_callback(self, path, arg):
        global receivingMessages
        receivingMessages = True
        # TODO apparently this callback never gets called
        x = int(arg) if not math.isnan(arg[0]) else 0
        self.state.touching_forehead = x

    # horseshoe gives more granular information on which contacts have signal
    # from the brain
    # We get a tuple of 4 numbers, each representing the connectivity of
    # each contact 1 = good, 2 = ok, >=3 bad
    @make_method('/muse/elements/horseshoe', 'ffff')
    def horseshoe_callback(self, path, args):
        global receivingMessages
        receivingMessages = True
        # A score between 0 and 1 of how good the connections of the contacts are
        connectionScore = (8 - (sum(map(lambda x: x if x <= 3 else 3, args)) - 4)) / 8.0
        self.state.connectionScore = connectionScore

        if self.all_contacts_mean.next(connectionScore, printDebug=True) < 0.05 and self.state.connected:
            # It has been at least CONTACT_LOS_TIMEOUT seconds of total LOS on all contacts
            self.state.connected = 0
            print "LOST CONNECTION"

        if connectionScore > 0.5 and not self.state.connected:
            # This is the first time the user has put the muse on in at least CONTACT_LOS_TIMEOUT second
            print "CONNECTED!!"
            self.state.connected = 1

        # logging
        self.connections_debug = args
    #receive accelrometer data
    # @make_method('/muse/acc', 'fff')
    # def acc_callback(self, path, args):
    #     acc_x, acc_y, acc_z = args
    # print "%s %f %f %f" % (path, acc_x, acc_y, acc_z)

    #receive jaw clench
    # @make_method('/muse/elements/jaw_clench', 'f')
    # def jaw_clench_callback(self, path, args):
    #     x = args
    #     print "%s, %f" % (path, x[0])

    #receive blink
    # @make_method('/muse/elements/blink', 'f')
    # def blink_callback(self, path, args):
    #     x = args
    #     print "%s, %f" % (path, x[0])

    #receive EEG data
    # @make_method('/muse/eeg', 'ffff')
    # def eeg_callback(self, path, args):
    #     l_ear, l_forehead, r_forehead, r_ear = args
    #     print "%s %f %f %f %f" % (path, l_ear, l_forehead, r_forehead, r_ear)

    #handle unexpected messages
    # @make_method(None, None)
    # def fallback(self, path, args, types, src):
        # print "Unknown message \
        # \n\t Source: '%s' \
        # \n\t Address: '%s' \
        # \n\t Types: '%s ' \
        # \n\t Payload: '%s'" \
        # % (src.url, path, types, args)

try:
    server = MuseServer()
except ServerError, err:
    print str(err)
    sys.exit()

server.start()

if __name__ == "__main__":
    while 1:
        try:
            time.sleep(1)
            server
        except KeyboardInterrupt:
            server.kill()
            sys.exit()




# How often we update the lights. Measured in seconds. Minimum of 0.1 (You can go lower, but we only get data from the muse at 10Hz)
LIGHT_UPDATE_INTERVAL = 0.01
# How often we update the nanoleaf lights. Measured in seconds. Because nanoleaf does smooth transitions by itself, we don't have to have such a high fps
NANOLEAF_LIGHT_UPDATE_INTERVAL = 0.1
# How often we internally render an new frame of the default animation
DEFAULT_ANIMATION_RENDER_RATE = 0.01
# Number of second over which we average eeg signals.
ROLLING_EEG_WINDOW = 6
# Number of second over which fade between user input and the default light animation.
USER_TO_DEFAULT_FADE_WINDOW = 3
# The delay in seconds between loss of signal on all contacts and ..doing something about it
CONTACT_LOS_TIMEOUT = 3
# the default brightness of the lights when the user is connected
USER_LIGHT_BRIGHTNESS = 125
# the default brightness of the Default animation
DEFAULT_COLOR_ANIMATION_BRIGHTNESS = 255
DEFAULT_SPOTLIGHT_ANIMATION_BRIGHTNESS = 125
DEFAULT_SPOTLIGHT_ANIMATION_BRIGHTNESS_RANGE = 50

# Light group addresses
EEG_LIGHT_GROUP_ADDRESS= 1
SPOTLIGHT_LIGHT_GROUP_ADDRESS = 8

# How often to print the log message in seconds
LOG_PRINT_RATE = 1

# Correct decimal place for relevant values. XXX Don't change me!
ROLLING_EEG_WINDOW *= 10
CONTACT_LOS_TIMEOUT *= 10
USER_TO_DEFAULT_FADE_WINDOW = USER_TO_DEFAULT_FADE_WINDOW / LIGHT_UPDATE_INTERVAL or 1



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="muse_nanoleaf",
    version="0.0.1",
    author='Lyon Lay',
    author_email='lay.lyon@gmail.com',
    description='A muse/nanoleaf integration',
    url='https://github.com/LLay/muse_nanoleaf',
    install_requires=['nanoleaf'],
    packages=setuptools.find_packages(),
    zip_safe=False
)



import sys
sys.path.insert(0, '/Users/lay/workspace/dmx-lib/muse_controller/nanoleafpy2')
from nanoleaf import setup, Aurora

# TODO this file needs to be automated

print "Retrieving auroras..."
ipAddressList = setup.find_auroras()
print "Auroras found:", ipAddressList

ipAddressList = [u'192.168.128.31', u'192.168.128.19', u'192.168.128.32']
print "Selecting only whitelisted auroras..."
ipWhitelist = []
ipAddressList = [x for x in ipAddressList if x in ipWhitelist]
print "Auroras used:", ipAddressList

print "Retrieving auth tokens..."
ipAuthMap = {}
for ip in ipAddressList:
    token = setup.generate_auth_token(ip)
    print "considering token", token
    if token is not None:
        ipAuthMap[ip] = token
print "Tokens generated:", ipAuthMap

ipAuthMap = {
    u'192.168.128.31': u'jcjYtfzt4zusn82lSJ5eHLOPzuiOgoDP',
    u'192.168.128.32': u'WTq3eIKY8fdfgGZjQlvbbe7E7fkKYDW4'
}
print "Instatiating Aurora Objects..."
auroras = []
for ip in ipAuthMap:
    auroras += [{'aurora': Aurora(ip, ipAuthMap[ip])}]
print "Finished Instatiating Aurora Objects..."

print "Initiating Aurora Metadata..."
for aurora in auroras:
    aurora['panelIDs'] = [x['panelId'] for x in aurora['aurora'].panel_positions]
    # print "aura panel meta data", aurora['panelIDs']
print "Finished Initiating Aurora Metadata..."

print "Turning on Auroras"
for aurora in auroras:
    aurora['aurora'].on = True
    # aurora['aurora'].effect = ANIMATION_ID
print "Finished Turning on Auroras"

# print "Setting lights to white"
# updateLights(255, 30, 255, 100)
# print "Finished setting lights to white"

# print "Identifying Auroras..."
# for aurora in auroras:
#     aurora['aurora'].identify()
